@@ -35,8 +35,10 @@
 #include "gnome-vfs-i18n.h"
 #include "gnome-vfs-private-utils.h"
 #include "gnome-vfs-ops.h"
+#include "gnome-vfs-mime-handlers.h"
 #include <glib/gstrfuncs.h>
 #include <glib/gutils.h>
+#include <gconf/gconf-client.h>
 #include <pwd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -58,7 +60,6 @@
 #define MEGABYTE_FACTOR (1024.0 * 1024.0)
 #define GIGABYTE_FACTOR (1024.0 * 1024.0 * 1024.0)
 
-
 #define READ_CHUNK_SIZE 8192
 
 #define MAX_SYMLINKS_FOLLOWED 32
@@ -773,7 +774,7 @@ gnome_vfs_get_volume_free_space (const GnomeVFSURI *vfs_uri,
        	int statfs_result;
 	const char *path, *scheme;
 	char *unescaped_path;
-
+	GnomeVFSResult ret;
 #if HAVE_STATVFS
 	struct statvfs statfs_buffer;
 #else
@@ -787,6 +788,8 @@ gnome_vfs_get_volume_free_space (const GnomeVFSURI *vfs_uri,
 		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 	}
 
+	unescaped_path = gnome_vfs_unescape_string (path, G_DIR_SEPARATOR_S);
+	
 	scheme = gnome_vfs_uri_get_scheme (vfs_uri);
 	
         /* We only handle the file scheme for now */
@@ -794,46 +797,27 @@ gnome_vfs_get_volume_free_space (const GnomeVFSURI *vfs_uri,
 		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 	}
 
-	unescaped_path = gnome_vfs_unescape_string (path, G_DIR_SEPARATOR_S);
-	
 #if HAVE_STATVFS
 	statfs_result = statvfs (unescaped_path, &statfs_buffer);
 #else
 	statfs_result = statfs (unescaped_path, &statfs_buffer);   
 #endif  
 
-	g_free (unescaped_path);
-
-	if (statfs_result != 0) {
-		return gnome_vfs_result_from_errno ();
+	if (statfs_result == 0) {
+		ret = GNOME_VFS_OK;
+	} else {
+		ret = gnome_vfs_result_from_errno ();
 	}
 	
+	g_return_val_if_fail (statfs_result == 0, FALSE);
 	block_size = statfs_buffer.f_bsize; 
 	free_blocks = statfs_buffer.f_bavail;
 
 	*size = block_size * free_blocks;
+
+	g_free (unescaped_path);
 	
-	return GNOME_VFS_OK;
-}
-
-/**
- * hack_file_exists
- * @filename: pathname to test for existance.
- *
- * Returns true if filename exists
- */
-/* FIXME: Why is this here? Why not use g_file_exists in libgnome/gnome-util.h?
- * (I tried to simply replace but there were strange include dependencies, maybe
- * that's why this function exists.)
- */
-static int
-hack_file_exists (const char *filename)
-{
-	struct stat s;
-
-	g_return_val_if_fail (filename != NULL,FALSE);
-    
-	return stat (filename, &s) == 0;
+	return ret;
 }
 
 char *
@@ -844,7 +828,7 @@ gnome_vfs_icon_path_from_filename (const char *relative_filename)
 	char **paths, **temp_paths;
 
 	if (g_path_is_absolute (relative_filename) &&
-	    hack_file_exists (relative_filename))
+	    g_file_test (relative_filename, G_FILE_TEST_EXISTS))
 		return g_strdup (relative_filename);
 
 	gnome_var = g_getenv ("GNOME_PATH");
@@ -857,7 +841,7 @@ gnome_vfs_icon_path_from_filename (const char *relative_filename)
 
 	for (temp_paths = paths; *temp_paths != NULL; temp_paths++) {
 		full_filename = g_strconcat (*temp_paths, "/share/pixmaps/", relative_filename, NULL);
-		if (hack_file_exists (full_filename)) {
+		if (g_file_test (full_filename, G_FILE_TEST_EXISTS)) {
 			g_strfreev (paths);
 			return full_filename;
 		}
@@ -1345,6 +1329,70 @@ gnome_vfs_make_uri_from_input (const char *location)
 	return gnome_vfs_make_uri_from_input_internal (location, broken_filenames, TRUE);
 }
 
+/**
+ * gnome_vfs_make_uri_from_input_with_dirs:
+ * @in: a relative or absolute path
+ *
+ * Determines a fully qualified URL from a relative or absolute input path.
+ * Basically calls gnome_vfs_make_uri_from_input except it specifically
+ * tries to support paths relative to the specified directories (can be homedir
+ * and/or current directory).
+ *
+ * Return value: the fully qualified URL
+ *
+ * Since: 2.4
+ */
+char *
+gnome_vfs_make_uri_from_input_with_dirs (const char *in,
+					 GnomeVFSMakeURIDirs dirs)
+{
+	char *uri, *path, *dir;
+
+	switch (in[0]) {
+	case '\0':
+		uri = g_strdup ("");
+		break;
+		
+	case '~':
+	case '/':
+		uri = gnome_vfs_make_uri_from_input (in);
+		break;
+		
+	default:
+		/* this might be a relative path, check if it exists relative
+		 * to current dir and home dir.
+		 */
+		uri = NULL;
+		if (dirs & GNOME_VFS_MAKE_URI_DIR_CURRENT) {
+			dir = g_get_current_dir ();
+			path = g_build_filename (dir, in, NULL);
+			g_free (dir);
+			
+			if (g_file_test (path, G_FILE_TEST_EXISTS)) {
+				uri = gnome_vfs_make_uri_from_input (path);
+			}
+			g_free (path);
+		}
+
+		if (uri == NULL &&
+		    dirs & GNOME_VFS_MAKE_URI_DIR_HOMEDIR) {
+			path = g_build_filename (g_get_home_dir (), in, NULL);
+		
+			if (g_file_test (path, G_FILE_TEST_EXISTS)) {
+				uri = gnome_vfs_make_uri_from_input (path);
+			}
+			g_free (path);
+		}
+
+		if (uri == NULL) {
+			uri = gnome_vfs_make_uri_from_input (in);
+		}
+	}
+	
+	return uri;
+}
+
+
 /**
  * gnome_vfs_make_uri_canonical_strip_fragment:
  * @uri:
@@ -1665,7 +1713,7 @@ file_uri_from_local_relative_path (const char *location)
 
 	location_escaped = gnome_vfs_escape_path_string (location);
 
-	uri = gnome_vfs_make_uri_full_from_relative (base_uri_slash, location_escaped);
+	uri = gnome_vfs_uri_make_full_from_relative (base_uri_slash, location_escaped);
 
 	g_free (location_escaped);
 	g_free (base_uri_slash);
@@ -1715,108 +1763,15 @@ gnome_vfs_make_uri_from_shell_arg (const char *location)
 	return uri;
 }
 
-static gboolean
-is_uri_partial (const char *uri)
-{
-	const char *current;
-
-	/* RFC 2396 section 3.1 */
-	for (current = uri ; 
-		*current
-		&& 	((*current >= 'a' && *current <= 'z')
-			 || (*current >= 'A' && *current <= 'Z')
-			 || (*current >= '0' && *current <= '9')
-			 || ('-' == *current)
-			 || ('+' == *current)
-			 || ('.' == *current)) ;
-	     current++);
-
-	return  !(':' == *current);
-}
-
-/*
- * FIXME this is not the simplest or most time-efficent way
- * to do this.  Probably a far more clear way of doing this processing
- * is to split the path into segments, rather than doing the processing
- * in place.
- */
-static void
-remove_internal_relative_components (char *uri_current)
-{
-	char *segment_prev, *segment_cur;
-	size_t len_prev, len_cur;
-
-	len_prev = len_cur = 0;
-	segment_prev = NULL;
-
-	g_return_if_fail (uri_current != NULL);
-
-	segment_cur = uri_current;
-
-	while (*segment_cur) {
-		len_cur = strcspn (segment_cur, "/");
-
-		if (len_cur == 1 && segment_cur[0] == '.') {
-			/* Remove "." 's */
-			if (segment_cur[1] == '\0') {
-				segment_cur[0] = '\0';
-				break;
-			} else {
-				memmove (segment_cur, segment_cur + 2, strlen (segment_cur + 2) + 1);
-				continue;
-			}
-		} else if (len_cur == 2 && segment_cur[0] == '.' && segment_cur[1] == '.' ) {
-			/* Remove ".."'s (and the component to the left of it) that aren't at the
-			 * beginning or to the right of other ..'s
-			 */
-			if (segment_prev) {
-				if (! (len_prev == 2
-				       && segment_prev[0] == '.'
-				       && segment_prev[1] == '.')) {
-				       	if (segment_cur[2] == '\0') {
-						segment_prev[0] = '\0';
-						break;
-				       	} else {
-						memmove (segment_prev, segment_cur + 3, strlen (segment_cur + 3) + 1);
-
-						segment_cur = segment_prev;
-						len_cur = len_prev;
-
-						/* now we find the previous segment_prev */
-						if (segment_prev == uri_current) {
-							segment_prev = NULL;
-						} else if (segment_prev - uri_current >= 2) {
-							segment_prev -= 2;
-							for ( ; segment_prev > uri_current && segment_prev[0] != '/' 
-							      ; segment_prev-- );
-							if (segment_prev[0] == '/') {
-								segment_prev++;
-							}
-						}
-						continue;
-					}
-				}
-			}
-		}
-
-		/*Forward to next segment */
-
-		if (segment_cur [len_cur] == '\0') {
-			break;
-		}
-		 
-		segment_prev = segment_cur;
-		len_prev = len_cur;
-		segment_cur += len_cur + 1;	
-	}	
-}
-
 /**
  * gnome_vfs_make_uri_full_from_relative:
  * 
  * Returns a full URI given a full base URI, and a secondary URI which may
  * be relative.
  *
+ * This function is deprecated, please use 
+ * gnome_vfs_uri_make_full_from_relative from gnome-vfs-uri.h
+ *
  * Return value: the URI (NULL for some bad errors).
  *
  * Since: 2.2
@@ -1825,128 +1780,7 @@ remove_internal_relative_components (char *uri_current)
 char *
 gnome_vfs_make_uri_full_from_relative (const char *base_uri, const char *relative_uri)
 {
-	char *result = NULL;
-
-	/* See section 5.2 in RFC 2396 */
-
-	if (base_uri == NULL && relative_uri == NULL) {
-		result = NULL;
-	} else if (base_uri == NULL) {
-		result = g_strdup (relative_uri);
-	} else if (relative_uri == NULL) {
-		result = g_strdup (base_uri);
-	} else if (!is_uri_partial (relative_uri)) {
-		result = g_strdup (relative_uri);
-	} else {
-		char *mutable_base_uri;
-		char *mutable_uri;
-
-		char *uri_current;
-		size_t base_uri_length;
-		char *separator;
-
-		mutable_base_uri = g_strdup (base_uri);
-		uri_current = mutable_uri = g_strdup (relative_uri);
-
-		/* Chew off Fragment and Query from the base_url */
-
-		separator = strrchr (mutable_base_uri, '#'); 
-
-		if (separator) {
-			*separator = '\0';
-		}
-
-		separator = strrchr (mutable_base_uri, '?');
-
-		if (separator) {
-			*separator = '\0';
-		}
-
-		if ('/' == uri_current[0] && '/' == uri_current [1]) {
-			/* Relative URI's beginning with the authority
-			 * component inherit only the scheme from their parents
-			 */
-
-			separator = strchr (mutable_base_uri, ':');
-
-			if (separator) {
-				separator[1] = '\0';
-			}			  
-		} else if ('/' == uri_current[0]) {
-			/* Relative URI's beginning with '/' absolute-path based
-			 * at the root of the base uri
-			 */
-
-			separator = strchr (mutable_base_uri, ':');
-
-			/* g_assert (separator), really */
-			if (separator) {
-				/* If we start with //, skip past the authority section */
-				if ('/' == separator[1] && '/' == separator[2]) {
-					separator = strchr (separator + 3, '/');
-					if (separator) {
-						separator[0] = '\0';
-					}
-				} else {
-				/* If there's no //, just assume the scheme is the root */
-					separator[1] = '\0';
-				}
-			}
-		} else if ('#' != uri_current[0]) {
-			/* Handle the ".." convention for relative uri's */
-
-			/* If there's a trailing '/' on base_url, treat base_url
-			 * as a directory path.
-			 * Otherwise, treat it as a file path, and chop off the filename
-			 */
-
-			base_uri_length = strlen (mutable_base_uri);
-			if ('/' == mutable_base_uri[base_uri_length-1]) {
-				/* Trim off '/' for the operation below */
-				mutable_base_uri[base_uri_length-1] = 0;
-			} else {
-				separator = strrchr (mutable_base_uri, '/');
-				if (separator) {
-					*separator = '\0';
-				}
-			}
-
-			remove_internal_relative_components (uri_current);
-
-			/* handle the "../"'s at the beginning of the relative URI */
-			while (0 == strncmp ("../", uri_current, 3)) {
-				uri_current += 3;
-				separator = strrchr (mutable_base_uri, '/');
-				if (separator) {
-					*separator = '\0';
-				} else {
-					/* <shrug> */
-					break;
-				}
-			}
-
-			/* handle a ".." at the end */
-			if (uri_current[0] == '.' && uri_current[1] == '.' 
-			    && uri_current[2] == '\0') {
-
-			    	uri_current += 2;
-				separator = strrchr (mutable_base_uri, '/');
-				if (separator) {
-					*separator = '\0';
-				}
-			}
-
-			/* Re-append the '/' */
-			mutable_base_uri [strlen(mutable_base_uri)+1] = '\0';
-			mutable_base_uri [strlen(mutable_base_uri)] = '/';
-		}
-
-		result = g_strconcat (mutable_base_uri, uri_current, NULL);
-		g_free (mutable_base_uri); 
-		g_free (mutable_uri); 
-	}
-	
-	return result;
+	return gnome_vfs_uri_make_full_from_relative (base_uri, relative_uri);
 }
 
 GnomeVFSResult
@@ -2111,4 +1945,91 @@ _gnome_vfs_uri_is_in_subdir (GnomeVFSURI *uri, GnomeVFSURI *dir)
 	return is_in_dir;
 }
 
-			  
+/**
+ * gnome_vfs_url_show:
+ * 
+ * Launches the default application or component associated with the given url.
+ *
+ * Return value: GNOME_VFS_OK if the default action was launched,
+ * GNOME_VFS_ERROR_BAD_PARAMETERS for an invalid or non-existant url,
+ * GNOME_VFS_ERROR_NOT_SUPPORTED if no default action is associated with the URL.
+ * Also error codes from gnome_vfs_mime_action_launch and
+ * gnome_vfs_url_show_using_handler for other errors.
+ *
+ * Since: 2.4
+ */
+GnomeVFSResult
+gnome_vfs_url_show (const char *url)
+{
+	return gnome_vfs_url_show_with_env (url, NULL);
+}
+
+/**
+ * gnome_vfs_url_show_with_env:
+ * 
+ * Like gnome_vfs_url_show except that the default action will be launched
+ * with the given environment.
+ *
+ * Return value: GNOME_VFS_OK if the default action was launched.
+ *
+ * Since: 2.4
+ */
+GnomeVFSResult
+gnome_vfs_url_show_with_env (const char  *url,
+                             char       **envp)
+{
+	GnomeVFSMimeApplication *app;
+	GnomeVFSMimeAction *action;
+	GnomeVFSResult result;
+	GList params;
+	char *type;
+	char *scheme;
+
+	g_return_val_if_fail (url != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+
+	scheme = gnome_vfs_get_uri_scheme (url);
+	if (scheme == NULL) {
+		return GNOME_VFS_ERROR_BAD_PARAMETERS;
+	}
+	
+	/* check if this scheme requires special handling */
+	if (_gnome_vfs_use_handler_for_scheme (scheme)) {
+		result = _gnome_vfs_url_show_using_handler_with_env (url, envp);
+		g_free (scheme);
+		return result;
+	}
+	
+	g_free (scheme);
+
+	type = gnome_vfs_get_mime_type (url);
+
+	if (type == NULL) {
+		return GNOME_VFS_ERROR_BAD_PARAMETERS;
+	}
+
+	params.data = (char *) url;
+	params.prev = NULL;
+	params.next = NULL;
+	
+	app = gnome_vfs_mime_get_default_application (type);
+	
+	if (app != NULL) {
+		result = gnome_vfs_mime_application_launch_with_env (app, &params, envp);
+		gnome_vfs_mime_application_free (app);
+		g_free (type);
+		return result;
+	}
+	
+	action = gnome_vfs_mime_get_default_action (type);
+	
+	if (action != NULL) {
+		result = gnome_vfs_mime_action_launch_with_env (action, &params, envp);
+		gnome_vfs_mime_action_free (action);
+		g_free (type);
+		return result;
+	}
+	
+	g_free (type);
+	return GNOME_VFS_ERROR_NO_DEFAULT;	
+}	  
+
