@@ -36,6 +36,9 @@
 #include <libgnomevfs/gnome-vfs-directory.h>
 #include <libgnomevfs/gnome-vfs-find-directory.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-ssl.h>
+#include <libgnomevfs/gnome-vfs-module-callback.h>
+#include <libgnomevfs/gnome-vfs-standard-callbacks.h>
 #include <popt.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -223,6 +226,8 @@ list_commands (void)
 	printf (" * cat,type:               dump text file to console\n");
 	printf (" * dump:                   dump binary file to console\n");
 	printf (" * sync:                   for sinkers\n");
+	printf (" * ssl:                    displays ssl enabled state\n");
+	printf (" * findtrash:              locates a trash directory for a URI\n");
 	printf (" * quit,exit,bye:          exit\n");
 	printf ("File operations:\n");
 	printf (" * open <handle> <name>:   open a file\n");
@@ -541,11 +546,21 @@ do_findtrash (void)
 	gnome_vfs_uri_unref (result_vfs_uri);
 }
 
+static void
+do_ssl (void)
+{
+	if (gnome_vfs_ssl_enabled ())
+		fprintf (stdout, "SSL enabled\n");
+	else
+		fprintf (stdout, "SSL disabled\n");
+}
+
 static void
 print_info (GnomeVFSFileInfo *info)
 {
 	const char *mime_type;
 	struct tm *loctime;
+
 	fprintf (stdout, "Name: '%s'\n", info->name);
 	if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_TYPE) {
 		fprintf (stdout, "Type: ");
@@ -943,6 +958,35 @@ callback (GIOChannel *source,
 	return TRUE;
 }
 
+static void
+authentication_callback (gconstpointer in, size_t in_size,
+			 gpointer out, size_t out_size,
+			 gpointer user_data)
+{
+	char buffer[512];
+ 	GnomeVFSModuleCallbackAuthenticationIn *in_real;
+ 	GnomeVFSModuleCallbackAuthenticationOut *out_real;
+
+ 	g_return_if_fail (sizeof (GnomeVFSModuleCallbackAuthenticationIn) == in_size
+ 		&& sizeof (GnomeVFSModuleCallbackAuthenticationOut) == out_size);
+
+	g_return_if_fail (in != NULL);
+	g_return_if_fail (out != NULL);
+
+ 	in_real = (GnomeVFSModuleCallbackAuthenticationIn *)in;
+ 	out_real = (GnomeVFSModuleCallbackAuthenticationOut *)out;
+	
+	printf ("Authenticate for uri: %s realm: %s\n", in_real->uri, in_real->realm);
+
+	printf ("Username:\n");
+	fgets (buffer, 511, stdin);
+	out_real->username = g_strndup (buffer, 512);
+
+	printf ("Password:\n");
+	fgets (buffer, 511, stdin);
+ 	out_real->password = g_strndup (buffer, 512);
+}
+
 int
 main (int argc, const char **argv)
 {
@@ -969,6 +1013,9 @@ main (int argc, const char **argv)
 		fprintf (vfserr, "Cannot initialize gnome-vfs.\n");
 		return 1;
 	}
+	gnome_vfs_module_callback_push
+		(GNOME_VFS_MODULE_CALLBACK_AUTHENTICATION,
+		 authentication_callback, NULL, NULL);
 
 	instream = stdin;
 	args = poptGetArgs (popt_context);
@@ -1054,6 +1101,8 @@ main (int argc, const char **argv)
 			do_info ();
 		else if (g_ascii_strcasecmp (ptr, "findtrash") == 0)
 			do_findtrash ();
+		else if (g_ascii_strcasecmp (ptr, "ssl") == 0)
+			do_ssl ();
 		else if (g_ascii_strcasecmp (ptr, "sync") == 0)
 			fprintf (vfserr, "a shell is like a boat, it lists or syncs (RMS)\n");
 		else if (g_ascii_strcasecmp (ptr,"help") == 0 ||
@@ -1095,3 +1144,4 @@ main (int argc, const char **argv)
 
 	return 0;
 }
+
